using UnityEngine;
using UnityEngine.UI;
using System;
using System.Collections;
using System.Collections.Generic;
using SimpleJSON;
using System.Linq;
using System.IO;
using MeshVR;
using Request = MeshVR.AssetLoader.AssetBundleFromFileRequest;
using AssetBundles;
using MVR;

// IciclesGlobalEditor v1.0
//
// Allows modifications and control of all Icicles CUA in a scene

namespace IciclesGlobalEditorPlugin
{
    public class IciclesGlobalEditor : MVRScript
    {
        public static string ASSETS_PATH;
	
		// JSONStorables
		private JSONStorableStringChooser editListChoice;
		
		public JSONStorableColor IceColor;
		public JSONStorableStringChooser IceTexture;
		public JSONStorableFloat IceSmoothness;
		public JSONStorableFloat IceSurfaceOffset;
		public JSONStorableFloat IceSurfaceIntensity;
		public JSONStorableFloat IceSurfaceOffsetIntensity;
		public JSONStorableFloat IceTextureSize;
		public JSONStorableFloat IceTextureScale;
		
		public JSONStorableColor IcewaterColor;
		public JSONStorableStringChooser IcewaterTexture;
		public JSONStorableFloat IcewaterOpacity;
		public JSONStorableFloat IcewaterSmoothness;
		public JSONStorableFloat IcewaterSurfaceOffset;
		public JSONStorableFloat IcewaterSurfaceIntensity;
		public JSONStorableFloat IcewaterSurfaceOffsetIntensity;
		public JSONStorableFloat IcewaterTextureSize;
		public JSONStorableFloat IcewaterTextureScale;
		public JSONStorableFloat IcewaterRefraction;
		
		public JSONStorableColor SnowrocksColor;
			
		// UI Elements
		private List<string> editList;
		private string editListDefault = "Ice";
		UIDynamic[] IceUIElems;
		UIDynamic[] IceWaterUIElems;
		UIDynamic[] SnowRocksUIElems;
			
		// Texts and miscs
		private JSONStorableString _helpText;
				
		// Security and initialization
		private bool IGEScanDone = false;

		// Material lists
		protected List<string> texturesChoices;
		private String[] IceMats;
		private String[] IceWaterMats;
		private String[] SnowRocksMats;
		
		// CUAS and Assets Lists and arrays
		private List<Transform> CUAsList;
		private Transform[] CUAsArray;
		private List<string> texturesPaths;
		
		// Assetbundle
		private Request contentBundleRequest = null;

		
		public override void Init()
        {			 
            try
            {
				if (containingAtom.type != "Empty")
                {
                    SuperController.LogError("Please add IciclesGlobalEditor on an Empty atom.");
                    return;
                }
				
				ASSETS_PATH = GetPluginPath(this);

				// Global help box
				_helpText = new JSONStorableString("Help",
					"<color=#000><size=35><b>Help - Icicle 101 - Material Editor</b></size></color>\n\n" + 
					"<color=#333>" +
					"This plugin allows you to customize all the materials of Icicles 101 in a scene.\n\n" +
					"Tweak the settings, you will see the change immediately on the assets. You will have to refresh the script if you add it before adding some new elements in the scene. So the best solution is to design the scene first and add the plugin at the end to tweak the appearance.\n\n" +
					"Change the material type at the top of the left column, you will access settings for each materials :\n\n" +
					"- Ice : Opaque rocks with a cool subsurface effect.\n\n" +
					"- Ice water : Transparent rocks like super clear frozen water.\n\n" +
					"- Snow rocks : Opaque stony rocks with snow on them.\n\n" +
					"</color>"
				);
				UIDynamic helpTextfield = CreateTextField(_helpText, true);
				helpTextfield.height = 1000.0f;
				
				// Creation of the material lists
				IceMats = new String[1];
				IceMats[0] = "M_ice01";
				
				IceWaterMats = new String[1];
				IceWaterMats[0] = "M_icewater01";
				
				SnowRocksMats = new String[5];
				SnowRocksMats[0] = "M_snowrock_01";
				SnowRocksMats[1] = "M_snowrock_02";
				SnowRocksMats[2] = "M_snowrock_03";
				SnowRocksMats[3] = "M_snowrock_04";
				SnowRocksMats[4] = "M_snowrock_05";
				
				
				texturesChoices = new List<string>();
				
				// UI Base
				editList = new List<string>();
				editList.Add("Ice");
				editList.Add("Ice Water");
				editList.Add("Snow Rocks");
				
				editListChoice = new JSONStorableStringChooser("materials", editList, editListDefault, "Material Type", (string choice)=>
				{
					ToggleUICallback(choice);
				}){ isStorable = false };
				UIDynamicPopup editTriggerPopup = CreateScrollablePopup(editListChoice);
				RegisterStringChooser(editListChoice);
				
				// UI Ice
				IceUIElems = new UIDynamic[8];
				
				HSVColor hsvcIce = HSVColorPicker.RGBToHSV(1f, 1f, 1f);
				IceColor = new JSONStorableColor("Ice Color", hsvcIce, IceColorCallback);
				IceUIElems[0] = CreateColorPicker(IceColor);
				
				IceTexture = new JSONStorableStringChooser("IceTexture", texturesChoices, "", "Ice Texture", IceStringCallback);
				IceUIElems[1] = CreateScrollablePopup(IceTexture, false);

				IceSmoothness = new JSONStorableFloat("Ice Smoothness", 0.3f, IceFloatCallback, 0f, 1f);
				IceUIElems[2] = CreateSlider(IceSmoothness);
				
				IceSurfaceOffset = new JSONStorableFloat("Ice Surface Offset", 600f, IceFloatCallback, 0f, 2000f);
				IceUIElems[3] = CreateSlider(IceSurfaceOffset);
				
				IceSurfaceIntensity = new JSONStorableFloat("Ice Surface Intensity", 0.2f, IceFloatCallback, 0.1f, 1f);
				IceUIElems[4] = CreateSlider(IceSurfaceIntensity);
				
				IceSurfaceOffsetIntensity = new JSONStorableFloat("Ice Surface Offset Intensity", 0.8f, IceFloatCallback, 0.1f, 2f);
				IceUIElems[5] = CreateSlider(IceSurfaceOffsetIntensity);
				
				IceTextureSize = new JSONStorableFloat("Ice Texture Size", 2048f, (val) => { IceTextureSize.valNoCallback = Mathf.Round(val); IceFloatCallback( IceTextureSize.valNoCallback ); }, 64f, 4096f);
				IceUIElems[6] = CreateSlider(IceTextureSize);
				
				IceTextureScale = new JSONStorableFloat("Ice Texture Scale", 4f, IceFloatCallback, 0.1f, 10f);
				IceUIElems[7] = CreateSlider(IceTextureScale);
				
				
				// UI Ice Water
				IceWaterUIElems = new UIDynamic[10];
				
				HSVColor hsvcIceWater = HSVColorPicker.RGBToHSV(1f, 1f, 1f);
				IcewaterColor = new JSONStorableColor("Ice Water Color", hsvcIceWater, IcewaterColorCallback);
				IceWaterUIElems[0] = CreateColorPicker(IcewaterColor);
				
				IcewaterTexture = new JSONStorableStringChooser("IcewaterTexture", texturesChoices, "", "Ice Water Texture", IcewaterStringCallback);
				IceWaterUIElems[1] = CreateScrollablePopup(IcewaterTexture, false);
				
				IcewaterOpacity = new JSONStorableFloat("Ice Water Opacity", 0.2f, IcewaterFloatCallback, 0f, 1f);
				IceWaterUIElems[2] = CreateSlider(IcewaterOpacity);
				
				IcewaterSmoothness = new JSONStorableFloat("Ice Water Smoothness", 0.3f, IcewaterFloatCallback, 0f, 1f);
				IceWaterUIElems[3] = CreateSlider(IcewaterSmoothness);
				
				IcewaterSurfaceOffset = new JSONStorableFloat("Ice Water Surface Offset", 800f, IcewaterFloatCallback, 0f, 2000f);
				IceWaterUIElems[4] = CreateSlider(IcewaterSurfaceOffset);
				
				IcewaterSurfaceIntensity = new JSONStorableFloat("Ice Water Surface Intensity", 0.3f,  IcewaterFloatCallback, 0.1f, 1f);
				IceWaterUIElems[5] = CreateSlider(IcewaterSurfaceIntensity);
				
				IcewaterSurfaceOffsetIntensity = new JSONStorableFloat("Ice Water Surface Offset Intensity", 0.5f,  IcewaterFloatCallback, 0.1f, 2f);
				IceWaterUIElems[6] = CreateSlider(IcewaterSurfaceOffsetIntensity);
				
				IcewaterTextureSize = new JSONStorableFloat("Ice Water Texture Size", 2048f,  (val) => { IcewaterTextureSize.valNoCallback = Mathf.Round(val); IcewaterFloatCallback( IcewaterTextureSize.valNoCallback ); }, 64f, 4096f);
				IceWaterUIElems[7] = CreateSlider(IcewaterTextureSize);
				
				IcewaterTextureScale = new JSONStorableFloat("Ice Water Texture Scale", 3f,  IcewaterFloatCallback, 0.1f, 10f);
				IceWaterUIElems[8] = CreateSlider(IcewaterTextureScale);
							
				IcewaterRefraction = new JSONStorableFloat("Ice Water Refraction", 0.9f, IcewaterFloatCallback, 0f, 1f);
				IceWaterUIElems[9] = CreateSlider(IcewaterRefraction);
				
				// UI Snow rocks
				SnowRocksUIElems = new UIDynamic[1];
				
				HSVColor hsvcSnowrocks = HSVColorPicker.RGBToHSV(1f, 1f, 1f);
				SnowrocksColor = new JSONStorableColor("Snow Rocks Color", hsvcSnowrocks, SnowrocksColorCallback);
				SnowRocksUIElems[0] = CreateColorPicker(SnowrocksColor);


				// Registering JSONStorables			
				RegisterColor(IceColor);
				RegisterStringChooser(IceTexture);
				RegisterFloat(IceSmoothness);
				RegisterFloat(IceSurfaceOffset);
				RegisterFloat(IceSurfaceIntensity);
				RegisterFloat(IceSurfaceOffsetIntensity);
				RegisterFloat(IceTextureSize);
				RegisterFloat(IceTextureScale);
				
				RegisterColor(IcewaterColor);
				RegisterStringChooser(IcewaterTexture);
				RegisterFloat(IcewaterOpacity);
				RegisterFloat(IcewaterSmoothness);
				RegisterFloat(IcewaterSurfaceOffset);
				RegisterFloat(IcewaterSurfaceIntensity);
				RegisterFloat(IcewaterSurfaceOffsetIntensity);
				RegisterFloat(IcewaterTextureSize);
				RegisterFloat(IcewaterTextureScale);
				RegisterFloat(IcewaterRefraction);
				
				RegisterColor(SnowrocksColor);
				
				// Loading the bundle
				Request request = new AssetLoader.AssetBundleFromFileRequest {path = ASSETS_PATH + "/icicles101content_assets.assetbundle", callback = OnBundleLoaded};
				AssetLoader.QueueLoadAssetBundleFromFile(request);
				
				// Initializing UI Default State
				ToggleUICallback("Ice");
			}
            catch(Exception e)
            {
                SuperController.LogError("IciclesGlobalEditor - Exception caught: " + e);
            }
		}
		
		void Start()
        {
			
        }
		
		// Used for the initialization of the editor.
		// When it is initialized, there's nothing done every frame
		void Update()
        {
			// If every CUA atom has been scanned, then no more stuff to do in the update
			if( IGEScanDone == true ) return;
			// Waiting for the loading to finish, then applying the detection mode on CUA that have an asset with the root_throwable inside
			if( IGEScanDone == false && !SuperController.singleton.isLoading ) {
				try
				{
					CUAsList = new List<Transform>();
					foreach(Atom at in SuperController.singleton.GetAtoms() ) {
						if( at.type == "CustomUnityAsset" ) {
							Transform atTR = getCUAObject( at.reParentObject );
							Transform atChildTr = getChildRoot( atTR, "root_ice" );
							if( atChildTr != null ) {
								CUAsList.Add( atChildTr );
							}
						}
					}

					CUAsArray = CUAsList.ToArray();
					IGEScanDone = true;

					// Initializing assets (potentially already done by the assetbundle loader for the additional assets, but well... I can't really be sure of that :/)
					ChangeIceSettings();
					ChangeIcewaterSettings();
					ChangeSnowrocksSettings();
				}
				catch(Exception e)
				{
					SuperController.LogError("IciclesGlobalEditor - Exception caught while initializing: " + e);
				}
			}
		}
		
		void OnDisable() {

		}
		
		
        // Recursive function searching for a transform
        private Transform getCUAObject( Transform parent ) {
            foreach( Transform child in parent )
            {
				if( child.name == "object" ) {
					return child;
				} else {
					Transform childSearch = getCUAObject(child);
					if (childSearch != null) return childSearch;
				}
            }
        
            return null;
        }
		
		// Recursive function searching for a transform containing the string set in objectNameSearchString
		// It is used the first time for the initialization of the script
		// This script NEEDS to use a string "containing" your original because the cloning of the original asset will add "Clone" behind it.
		private Transform getObjectRoot( Transform parent, string rootName ) {
			foreach( Transform child in parent )
			{
				if( child.name.Contains(rootName) ) {
					return child;
				} else {
					Transform childSearch = getObjectRoot(child,rootName);
					if (childSearch != null) return childSearch;
				}
			}
			
			return null;
		}
		
		private Transform getChildRoot( Transform parent, string searchName ) {
			foreach( Transform child in parent )
			{
				if( child.name == searchName ) {
					return child;
				} else {
					Transform childSearch = getChildRoot(child,searchName);
					if (childSearch != null) return childSearch;
				}
			}
			
			return null;
		}

		protected void InitAssetsPaths() {
			// Creating assets paths list
			texturesPaths = new List<string>();
			texturesPaths.Add("Assets/VAMModels/Icicles101Content/textures/T_ice_core_01.png");
			texturesPaths.Add("Assets/VAMModels/Icicles101Content/textures/T_ice_core_02.png");
			texturesPaths.Add("Assets/VAMModels/Icicles101Content/textures/T_ice_core_03.png");
			texturesPaths.Add("Assets/VAMModels/Icicles101Content/textures/T_ice_core_04.png");
			texturesPaths.Add("Assets/VAMModels/Icicles101Content/textures/T_ice_core_05.png");
			texturesPaths.Add("Assets/VAMModels/Icicles101Content/textures/T_ice_core_06.png");

		}

		// **************************
		// Callbacks & similar
		// **************************
		private void OnBundleLoaded(Request aRequest) {
			try
            {
				InitAssetsPaths();
				
				contentBundleRequest = aRequest;

				texturesChoices = new List<string>();
				foreach( string path in texturesPaths ) {
					string[] aPath = path.Split('/');
					string[] aFile = aPath.Last().Split('.');
					string fileName = aFile[aFile.Length - 2].ToString();
					texturesChoices.Add( fileName );
				}
				
				IceTexture.choices = texturesChoices;
				IcewaterTexture.choices = texturesChoices;
				
				// Initializing assets
				ChangeIceSettings();
				ChangeIcewaterSettings();
				ChangeSnowrocksSettings();
			}
            catch(Exception e)
            {
                SuperController.LogError("VAMAAmbience - Exception caught: " + e);
            }
		}
		
		protected void ToggleUICallback( string type ) {
			// Hiding everything
			foreach( UIDynamic uidElem in IceUIElems ) {
				uidElem.GetComponent<LayoutElement>().transform.localScale = new Vector3(0,0,0);
				uidElem.GetComponent<LayoutElement>().ignoreLayout = true;
			}
					
			foreach( UIDynamic uidElem in IceWaterUIElems ) {
				uidElem.GetComponent<LayoutElement>().transform.localScale = new Vector3(0,0,0);
				uidElem.GetComponent<LayoutElement>().ignoreLayout = true;
			}
			
			foreach( UIDynamic uidElem in SnowRocksUIElems ) {
				uidElem.GetComponent<LayoutElement>().transform.localScale = new Vector3(0,0,0);
				uidElem.GetComponent<LayoutElement>().ignoreLayout = true;
			}
			
			// Showing only needed type
			switch( type ) {
				case "Ice":
					foreach( UIDynamic uidElem in IceUIElems ) {
						uidElem.GetComponent<LayoutElement>().transform.localScale = new Vector3(1,1,1);
						uidElem.GetComponent<LayoutElement>().ignoreLayout = false;
					}
					break;
				case "Ice Water":					
					foreach( UIDynamic uidElem in IceWaterUIElems ) {
						uidElem.GetComponent<LayoutElement>().transform.localScale = new Vector3(1,1,1);
						uidElem.GetComponent<LayoutElement>().ignoreLayout = false;
					}
					break;
				case "Snow Rocks":					
					foreach( UIDynamic uidElem in SnowRocksUIElems ) {
						uidElem.GetComponent<LayoutElement>().transform.localScale = new Vector3(1,1,1);
						uidElem.GetComponent<LayoutElement>().ignoreLayout = false;
					}
					break;
			}
		}
		
		protected void IceStringCallback(string choice) {
			// Not using the value, we're gonna get every JSONStorable values directly
			ChangeIceSettings();
		}
		
		protected void IceColorCallback(JSONStorableColor color) {
			// Not using the value, we're gonna get every JSONStorable values directly
			ChangeIceSettings();
		}
		
		protected void IceFloatCallback(float val) {
			// Not using the value, we're gonna get every JSONStorable values directly
			ChangeIceSettings();
		}
		
		protected void IcewaterStringCallback(string choice) {
			// Not using the value, we're gonna get every JSONStorable values directly
			ChangeIcewaterSettings();
		}
		
		protected void IcewaterColorCallback(JSONStorableColor color) {
			// Not using the value, we're gonna get every JSONStorable values directly
			ChangeIcewaterSettings();
		}
		
		protected void IcewaterFloatCallback(float val) {
			// Not using the value, we're gonna get every JSONStorable values directly
			ChangeIcewaterSettings();
		}
		
		protected void SnowrocksColorCallback(JSONStorableColor color) {
			// Not using the value, we're gonna get every JSONStorable values directly
			ChangeSnowrocksSettings();
		}
		
		// **************************
		// Functions
		// **************************
		protected void ChangeIceSettings() {
			if( CUAsArray == null || CUAsArray.Length <= 0 ) return;
			foreach( Transform child in CUAsArray )
			{
				foreach( Transform subChild in child ) {
					// Getting renderer
					Renderer objectRenderer = subChild.GetComponent<Renderer>();
					
					// Modifying only ice materials
					string[] matName = objectRenderer.material.name.Split(' ');
					if(objectRenderer != null && Array.IndexOf( IceMats, matName[0] ) != -1 ) {
						// Changing color
						Color newColor = new Color(IceColor.colorPicker.currentColor.r, IceColor.colorPicker.currentColor.g, IceColor.colorPicker.currentColor.b, 1f);
						objectRenderer.material.SetColor("_NoiseColor",newColor);
						
						// Changing noise texture
						if( IceTexture.val == "" ) IceTexture.val = "T_ice_core_02";
						if( contentBundleRequest != null ) {
							Texture2D iceTex = contentBundleRequest.assetBundle.LoadAsset<Texture2D>(texturesPaths[ texturesChoices.IndexOf(IceTexture.val) ]);
							objectRenderer.material.SetTexture("_NoiseTexture",iceTex);
						}

						// Changing smoothness
						objectRenderer.material.SetFloat("_Smoothness",IceSmoothness.val);
						
						// Changing surface offset
						objectRenderer.material.SetFloat("_SurfaceOffset",IceSurfaceOffset.val);
						
						// Changing surface intensity
						objectRenderer.material.SetFloat("_SurfaceIntensity", IceSurfaceIntensity.val);
						
						// Changing surface offset intensity
						objectRenderer.material.SetFloat("_OffsetIntensity", IceSurfaceOffsetIntensity.val);
						
						// Changing noise texture size
						objectRenderer.material.SetFloat("_NoiseTextureSize",IceTextureSize.val);
						
						// Changing noise texture scale
						objectRenderer.material.SetTextureScale("_SurfaceNormal", new Vector2(IceTextureScale.val, IceTextureScale.val));
					}
				}
			}
		}
		
		protected void ChangeIcewaterSettings() {
			if( CUAsArray == null || CUAsArray.Length <= 0 ) return;
			foreach( Transform child in CUAsArray )
			{
				foreach( Transform subChild in child ) {
					// Getting renderer
					Renderer objectRenderer = subChild.GetComponent<Renderer>();
					
					// Modifying only icewater materials
					string[] matName = objectRenderer.material.name.Split(' ');
					if(objectRenderer != null && Array.IndexOf( IceWaterMats, matName[0] ) != -1 ) {
						// Changing color
						Color newColor = new Color(IcewaterColor.colorPicker.currentColor.r, IcewaterColor.colorPicker.currentColor.g, IcewaterColor.colorPicker.currentColor.b, 1f);
						objectRenderer.material.SetColor("_NoiseColor",newColor);
						
						// Changing noise texture
						if( IcewaterTexture.val == "" ) IcewaterTexture.val = "T_ice_core_02";
						if( contentBundleRequest != null ) {
							Texture2D iceTex = contentBundleRequest.assetBundle.LoadAsset<Texture2D>(texturesPaths[ texturesChoices.IndexOf(IcewaterTexture.val) ]);
							objectRenderer.material.SetTexture("_NoiseTexture",iceTex);
						}
				
						// Changing opacity
						objectRenderer.material.SetFloat("_Opacity",IcewaterOpacity.val);

						// Changing smoothness
						objectRenderer.material.SetFloat("_Smoothness",IcewaterSmoothness.val);
						
						// Changing surface offset
						objectRenderer.material.SetFloat("_SurfaceOffset",IcewaterSurfaceOffset.val);

						// Changing surface intensity
						objectRenderer.material.SetFloat("_SurfaceIntensity", IcewaterSurfaceIntensity.val);
						
						// Changing surface offset intensity
						objectRenderer.material.SetFloat("_OffsetIntensity", IcewaterSurfaceOffsetIntensity.val);
						
						// Changing noise texture size
						objectRenderer.material.SetFloat("_NoiseTextureSize",IcewaterTextureSize.val);
						
						// Changing noise texture scale
						objectRenderer.material.SetTextureScale("_SurfaceNormal", new Vector2(IcewaterTextureScale.val, IcewaterTextureScale.val));
						
						// Changing refraction
						objectRenderer.material.SetFloat("_RefractionIntensity",IcewaterRefraction.val);
					}
				}
			}
		}
		
		protected void ChangeSnowrocksSettings() {
			if( CUAsArray == null || CUAsArray.Length <= 0 ) return;
			foreach( Transform child in CUAsArray )
			{
				foreach( Transform subChild in child ) {
					// Getting renderer
					Renderer objectRenderer = subChild.GetComponent<Renderer>();
					
					// Modifying only ice materials
					string[] matName = objectRenderer.material.name.Split(' ');
					if(objectRenderer != null && Array.IndexOf( SnowRocksMats, matName[0] ) != -1 ) {
						// Changing color
						Color newColor = new Color(SnowrocksColor.colorPicker.currentColor.r, SnowrocksColor.colorPicker.currentColor.g, SnowrocksColor.colorPicker.currentColor.b, 1f);
						objectRenderer.material.SetColor("_Color",newColor);
					}
				}
			}
		}
		
		// **************************
		// COROUTINES
		// **************************


		// **************************
		// Time to cleanup !
		// **************************		
		void OnDestroy() {			
			// Cleaning modified native VAM assets
			AssetLoader.DoneWithAssetBundleFromFile(ASSETS_PATH + "/icicles101content_assets.assetbundle");

		}
		
		// **************************
		// Local Tools
		// **************************
		private void logDebug( string debugText ) {
			SuperController.LogMessage( debugText );
		}
		
		private void logError( string debugText ) {
			SuperController.LogError( debugText );
		}

		// ***********************************************************
		// EXTERNAL TOOLS - Thank you great coders for your content!
		// ***********************************************************
		
		// *********** MacGruber_Utils.cs START *********************
		// Get directory path where the plugin is located. Based on Alazi's & VAMDeluxe's method.
		public static string GetPluginPath(MVRScript self)
		{
			string id = self.name.Substring(0, self.name.IndexOf('_'));
			string filename = self.manager.GetJSON()["plugins"][id].Value;
			return filename.Substring(0, filename.LastIndexOfAny(new char[] { '/', '\\' }));
		}
				
		// Get path prefix of the package that contains our plugin.
		public static string GetPackagePath(MVRScript self)
		{
			string id = self.name.Substring(0, self.name.IndexOf('_'));
			string filename = self.manager.GetJSON()["plugins"][id].Value;
			int idx = filename.IndexOf(":/");
			if (idx >= 0)
				return filename.Substring(0, idx+2);
			else
				return string.Empty;
		}
				
		// Check if our plugin is running from inside a package
		public static bool IsInPackage(MVRScript self)
		{
			string id = self.name.Substring(0, self.name.IndexOf('_'));
			string filename = self.manager.GetJSON()["plugins"][id].Value;
			return filename.IndexOf(":/") >= 0;
		}

	}
}