using UnityEngine;
using System;
using System.Collections;
using System.Collections.Generic;
using SimpleJSON;
using System.Linq;
using System.IO;

// Icicles101SnowEditor v1.0
//
// Allows modifications of snow vfx in Icicles 101

namespace Icicles101SnowEditorPlugin
{
    public class Icicles101SnowEditor : MVRScript
    {
		// JSONStorables
		public JSONStorableColor SnowColor;
		public JSONStorableFloat SnowRatePrt;
		public JSONStorableFloat SnowMaxPrt;
		
		public JSONStorableFloat SnowSizeMin;
		public JSONStorableFloat SnowSizeMax;
		
		public JSONStorableFloat SnowGravity;
		public JSONStorableFloat SnowWindX;
		public JSONStorableFloat SnowWindZ;
		public JSONStorableFloat SnowWindOrbitX;
		public JSONStorableFloat SnowWindOrbitY;
		public JSONStorableFloat SnowWindOrbitZ;
		
		public JSONStorableBool ShowVFXBounds;
		
		public JSONStorableFloat SnowAreaWidth;
		public JSONStorableFloat SnowAreaLength;
		public JSONStorableFloat SnowAreaHeight;
		
		// Object components
		public Transform ObjectRoot;
		public Transform VFXRoot;
		private ParticleSystem prtComponent;
		private ParticleSystemRenderer prtRendererComponent;

		private JSONStorableString _loadingInfos;
		private JSONStorableString _helpText;
		
		private GameObject debugParent;
		private GameObject boundsCube;
		
		// Security and initialization
		private bool editorInitialized = false;
		private int initLoops = 0;
		private int maxInitLoops = 3600;
		private bool cuaIsValid = false;
		

		
		// Spacers
		private UIDynamic spacer1;
		
		public override void Init()
        {			 
            try
            {
				if (containingAtom.type != "CustomUnityAsset")
                {
                    SuperController.LogError("Please add Icicles101SnowEditor on a compatible Custom Unity Asset.");
                    return;
                }
                
				// Global help box
				_helpText = new JSONStorableString("Help",
					"<color=#000><size=35><b>Help - Icicle 101 - Snow Editor</b></size></color>\n\n" + 
					"<color=#333>" +
					"This plugin allows you to customize the snow VFX in Icicles 101.\n\n" +
					"Every settings are pretty much self-explainatory. If you have any doubt, play with the sliders to see the visual effect update in realtime\n\n" +
					"For a realtime use, maximum particle count suggested is around 30k to 50k (for all CUAs, not PER CUA).\n\n" +
					"</color>"
				);
				
				// Bounds
				ShowVFXBounds = new JSONStorableBool("Show VFX bounds", false, ShowBoundsCallback);

				SnowAreaWidth = new JSONStorableFloat("Snow area width", 5f, VFXFloatCallback, 0.1f, 50f);	
				SnowAreaLength = new JSONStorableFloat("Snow area length", 5f, VFXFloatCallback, 0.1f, 50f);
				SnowAreaHeight = new JSONStorableFloat("Snow area height", 5f, VFXFloatCallback, 0.1f, 50f);
				
				// Settings UI
				HSVColor hsvc = HSVColorPicker.RGBToHSV(1f, 1f, 1f);
				SnowColor = new JSONStorableColor("Snow color", hsvc, VFXColorCallback);
				
				SnowRatePrt = new JSONStorableFloat("Snow particles rate", 2000f, (val) => { SnowRatePrt.valNoCallback = Mathf.Round(val); VFXFloatCallback(SnowRatePrt.valNoCallback); }, 100f, 20000f);
				SnowMaxPrt = new JSONStorableFloat("Snow max particles", 10000f, (val) => { SnowMaxPrt.valNoCallback = Mathf.Round(val); VFXFloatCallback(SnowMaxPrt.valNoCallback); }, 100f, 100000f);
				
				SnowSizeMin = new JSONStorableFloat("Snow Size (min)", 0.1f, VFXFloatCallback, 0.01f, 2f);
				SnowSizeMax = new JSONStorableFloat("Snow Size (max)", 0.6f, VFXFloatCallback, 0.01f, 2f);
				
				SnowGravity = new JSONStorableFloat("Snow Gravity", 0.01f, VFXFloatCallback, -1f, 1f);
				
				SnowWindX = new JSONStorableFloat("Wind Strength (Left/Right)", 0.01f, VFXFloatCallback, -1f, 1f);
				SnowWindZ = new JSONStorableFloat("Wind Strength (Front/Back)", 0.01f, VFXFloatCallback, -1f, 1f);
				
				SnowWindOrbitX = new JSONStorableFloat("Wind Orbit (Left/Right)", 0f, VFXFloatCallback, -1f, 1.000f);
				SnowWindOrbitY = new JSONStorableFloat("Wind Orbit (Up/Down)", 0f, VFXFloatCallback, -1f, 1f);
				SnowWindOrbitZ = new JSONStorableFloat("Wind Orbit (Front/Back)", 0f, VFXFloatCallback, -1f, 1f);


				// Registering Settings				
				RegisterColor(SnowColor);
				
				RegisterFloat(SnowRatePrt);
				RegisterFloat(SnowMaxPrt);
				
				RegisterFloat(SnowSizeMin);
				RegisterFloat(SnowSizeMax);
				
				RegisterFloat(SnowGravity);
				RegisterFloat(SnowWindX);
				RegisterFloat(SnowWindZ);
				RegisterFloat(SnowWindOrbitX);
				RegisterFloat(SnowWindOrbitY);
				RegisterFloat(SnowWindOrbitZ);
				
				RegisterFloat(SnowAreaWidth);
				RegisterFloat(SnowAreaLength);
				RegisterFloat(SnowAreaHeight);
				
				// Creating the default UI that will inform you that the thing is loading
				CreateLoadingUI();
				
				// ************************************************
				// ** ACTIONS
				// ************************************************			


			}
            catch(Exception e)
            {
                SuperController.LogError("Icicles101SnowEditor - Exception caught: " + e);
            }
		}
		
		void Start()
        {
			
        }
		
		// Used for the initialization of the editor.
		// When it is initialized, there's nothing done every frame
		void Update()
        {
			// If the editor is not initialized, we're gonna try to do it
			// I'm putting 3600 tries ( at 60fps it is 1min )
			if( initLoops < maxInitLoops && editorInitialized == false ) {
				initLoops++;
				// If our CUA is loading and everything is ok, we can remove the loading UI and Init the final UI
				if (InitEditor() == true)
				{
					RemoveLoadingUI();
					cuaIsValid = true; // Our CUA is initialized, we consider it as valid
					StartCoroutine("CheckCUAStatus"); // We're gonna check if the CUA is valid every 5 secs
				}
			}
			
			// Triggering an error if we have reached our limit of tries
			if( initLoops == maxInitLoops && editorInitialized == false ) {
				initLoops++; // To avoid spamming the log with the error
			}
		}
		
		void OnDisable() {

		}
		
		private bool InitEditor() {
			// Updating our loading UI
			UpdateLoadingUI();

			if( editorInitialized == true ) return true;
			
			ObjectRoot = getObjectRoot( containingAtom.reParentObject, "VFX_Snow01" );
			if( ObjectRoot == null )
			{
				return false;
			}

			VFXRoot = getChildRoot( ObjectRoot, "VFX_MainSystem" );
			if( VFXRoot != null ) {
				prtComponent = VFXRoot.GetComponent<ParticleSystem>();
				if( prtComponent != null ) {
					prtRendererComponent = VFXRoot.GetComponent<ParticleSystemRenderer>();
					if( prtRendererComponent == null ) return false;
				} else {
					return false;
				}
			} else {
				return false;
			}
			
			editorInitialized = true;
			CreateCUAUI();

			UpdateVFXSettings();
			
			return true;

		}
		
		// The function creating the loading UI before everything is ready
		private void CreateLoadingUI()
		{
			_loadingInfos = new JSONStorableString("Loading Infos", "");
			UIDynamic stateinfosTextfield = CreateTextField(_loadingInfos, false);
			stateinfosTextfield.height = 450.0f;
		}

		// The function clearing the loading UI
		private void RemoveLoadingUI()
		{
			// Removing the loading text field on the left of the UI
			RemoveTextField(_loadingInfos);
		}

		// The function updating the loading UI while we're either waiting for the CUA to load, or waiting for the root object and everything to be found
		private void UpdateLoadingUI()
		{
			var loadingText = "<color=#000><b>Custom Unity Asset loading...</b></color>\n" + (initLoops/60) + "/" + (maxInitLoops/60) + "secs";
			if (initLoops == maxInitLoops)
			{
				loadingText += "\n\n<color=#333><b>UNABLE TO LOAD YOUR CUA :</b>\n";
				loadingText += "- Your CUA may not be compatible with this script\n";
				loadingText += "- Your CUA may take too long to load</color>";
				loadingText += "\n\nSelect another CUA and reload the script, or just reload the script to try again.";
			}
			_loadingInfos.val = loadingText;
		}
		
		// The function creating the UI when the CUA has been invalidated
		private void CreateCUAInvalidatedUI()
		{
			JSONStorableString _invalidatedCUA = new JSONStorableString("Invalidated CUA", "");
			UIDynamic stateinfosTextfield = CreateTextField(_invalidatedCUA, false);
			stateinfosTextfield.height = 150.0f;
			_invalidatedCUA.val = "<color=#000><b>Custom Unity Asset invalidated.</b></color>\n" +
			                      "You have changed the CUA in the <b>Asset</b> tab, the script is no longer valid. Please reload it or change it.";
		}

		// Function creating the whole CUA UI
		private void CreateCUAUI()
		{
			CreateColorPicker(SnowColor);
			CreateSlider(SnowRatePrt);
			CreateSlider(SnowMaxPrt);
			
			CreateSlider(SnowSizeMin);
			CreateSlider(SnowSizeMax);
			
			CreateSlider(SnowGravity);
			CreateSlider(SnowWindX);
			CreateSlider(SnowWindZ);
			CreateSlider(SnowWindOrbitX);
			CreateSlider(SnowWindOrbitY);
			CreateSlider(SnowWindOrbitZ);
				
			UIDynamic helpTextfield = CreateTextField(_helpText, true);
			helpTextfield.height = 600.0f;

			CreateToggle(ShowVFXBounds, true);
			
			CreateSlider(SnowAreaWidth, true);
			CreateSlider(SnowAreaLength, true);
			CreateSlider(SnowAreaHeight, true);
		}
		
		// Function clearing the whole CUA UI
		private void ClearCUAUI()
		{
			RemoveColorPicker(SnowColor);
			RemoveSlider(SnowRatePrt);
			RemoveSlider(SnowMaxPrt);
			
			RemoveSlider(SnowSizeMin);
			RemoveSlider(SnowSizeMax);
			
			RemoveSlider(SnowGravity);
			RemoveSlider(SnowWindX);
			RemoveSlider(SnowWindZ);
			RemoveSlider(SnowWindOrbitX);
			RemoveSlider(SnowWindOrbitY);
			RemoveSlider(SnowWindOrbitZ);
			
			RemoveTextField(_helpText);
			
			RemoveToggle(ShowVFXBounds);
			RemoveSlider(SnowAreaWidth);
			RemoveSlider(SnowAreaLength);
			RemoveSlider(SnowAreaHeight);
		}
		
		// Recursive function searching for a transform containing the string set in objectNameSearchString
		// It is used the first time for the initialization of the script
		// This script NEEDS to use a string "containing" your original because the cloning of the original asset will add "Clone" behind it.
		private Transform getObjectRoot( Transform parent, string rootName ) {
			foreach( Transform child in parent )
			{
				if( child.name.Contains(rootName) ) {
					return child;
				} else {
					Transform childSearch = getObjectRoot(child,rootName);
					if (childSearch != null) return childSearch;
				}
			}
			
			return null;
		}
		
		private Transform getChildRoot( Transform parent, string searchName ) {
			foreach( Transform child in parent )
			{
				if( child.name == searchName ) {
					return child;
				} else {
					Transform childSearch = getChildRoot(child,searchName);
					if (childSearch != null) return childSearch;
				}
			}
			
			return null;
		}
		
		protected void UpdateVFXSettings() {
			if( VFXRoot != null ) {
				// All various components of the particle system
				var prtMain = prtComponent.main;
				var prtShape = prtComponent.shape;
				var prtVoL = prtComponent.velocityOverLifetime;
				var prtEmi = prtComponent.emission;
				
				// Scale of the particle system
				prtShape.scale = new Vector3(SnowAreaWidth.val,SnowAreaHeight.val,SnowAreaLength.val);
				UpdateBoundsCube();
				
				// Color
				if( SnowColor != null && SnowColor.colorPicker != null && SnowColor.colorPicker.currentColor != null ) {
					prtMain.startColor = new Color(SnowColor.colorPicker.currentColor.r, SnowColor.colorPicker.currentColor.g, SnowColor.colorPicker.currentColor.b, 1.0f);
				}
				
				// Max particles
				prtMain.maxParticles = (int)SnowMaxPrt.val;
				prtEmi.rateOverTime = SnowRatePrt.val;
				
				// Size
				prtMain.startSize = new ParticleSystem.MinMaxCurve(SnowSizeMin.val,SnowSizeMax.val);
				
				// Gravity
				prtMain.gravityModifier = SnowGravity.val;
				
				// Velocity
				prtVoL.x = new ParticleSystem.MinMaxCurve(SnowWindX.val,SnowWindX.val);
				prtVoL.z = new ParticleSystem.MinMaxCurve(SnowWindZ.val,SnowWindZ.val);

				// Velocity Orbital
				prtVoL.orbitalX = SnowWindOrbitX.val;
				prtVoL.orbitalY = SnowWindOrbitY.val;
				prtVoL.orbitalZ = SnowWindOrbitZ.val;
				
			}
		}
		
		
		protected void UpdateBoundsCube() {
			if ( boundsCube != null ) {
				boundsCube.transform.localScale = new Vector3( SnowAreaWidth.val, SnowAreaHeight.val, SnowAreaLength.val );
			}
		}
		
		// **************************
		// Callbacks & similar
		// **************************
		private void VFXColorCallback(JSONStorableColor color) {
			UpdateVFXSettings();
		}
		
		private void VFXFloatCallback(float val) {
			UpdateVFXSettings();
		}
		
		protected void ShowBoundsCallback(bool enabled) {
			if( prtComponent ) {
				if( debugParent == null ) {
					debugParent = new GameObject();
					debugParent.name = "hzmDebugParent";
					debugParent.transform.SetParent(ObjectRoot.parent.transform, false);
				}
				
				if( enabled == true ) {
					boundsCube = GameObject.CreatePrimitive(PrimitiveType.Cube);
					boundsCube.name = "hzmBoundsCube";
					Destroy(boundsCube.GetComponent<BoxCollider>());
					boundsCube.transform.SetParent(debugParent.transform,false);
					
					MeshRenderer mr = boundsCube.GetComponent<MeshRenderer>();
					mr.material = new Material(Shader.Find("Particles/Additive"));
					mr.material.SetColor("_TintColor", new Color(0.8f, 0.1f, 1f, 0.01f));
					
					UpdateBoundsCube();
				} else {
					GameObject.Destroy(debugParent);
					debugParent = null;
					boundsCube = null;
				}
			}
		}
		
		// **************************
		// COROUTINES
		// **************************
		
		// Routine checking every 5secs if our CUA is still valid, this will invalidate the UI if the user change the CUA in the asset tab
		private IEnumerator CheckCUAStatus()
		{
			while (cuaIsValid)
			{
				// Our object root is null, it means our CUA is invalid
				if (ObjectRoot == null)
				{
					// We invalidate it so that the coroutine stops by itself, remove the UI and show the UI to alert the user
					cuaIsValid = false;
					ClearCUAUI();
					CreateCUAInvalidatedUI();
				}
				yield return new WaitForSeconds(5.0f);
			}
		}

		// **************************
		// Time to cleanup !
		// **************************		
		void OnDestroy() {			
			// Cleaning debug
			if( debugParent != null ) {
				GameObject.Destroy(debugParent);
				debugParent = null;
				boundsCube = null;
			}
		}
		
		// **************************
		// Local Tools
		// **************************
		private void logDebug( string debugText ) {
			SuperController.LogMessage( debugText );
		}
		
		private void logError( string debugText ) {
			SuperController.LogError( debugText );
		}


	}
}